/*
 * @brief NXP LPC1769 XPresso board file
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 *
 * @note
 * modified Jordi Arroyo, 2016
 *
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 */

#ifndef __BOARD_H_
#define __BOARD_H_

#include "chip.h"
#include "board_api.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup BOARD_NXP_LPCXPRESSO_1769 NXP LPC1769 LPCXpresso board software API functions
 * @ingroup LPCOPEN_17XX_BOARD_LPCXPRESSO_1769
 * The board support software API functions provide some simple abstracted
 * functions used across multiple LPCOpen board examples. See @ref BOARD_COMMON_API
 * for the functions defined by this board support layer.<br>
 * @{
 */

/** @defgroup BOARD_NXP_LPCXPRESSO_1769_OPTIONS BOARD: NXP LPC1769 LPCXpresso board build options
 * This board has options that configure its operation at build-time.<br>
 * @{
 */

/** Board UART used for debug output and input using the DEBUG* macros. This
    is also the port used for Board_UARTPutChar, Board_UARTGetChar, and
    Board_UARTPutSTR functions.
 */
#define DEBUG_UART 			LPC_UART3
#define DEBUG_BAUDRATE 		115200

/** Wifly UART used for Wifly output and input using the WIFLY* macros. This
    is also the port used for Wifly_UARTPutChar, Wifly_UARTGetChar, and
    Wifly_UARTPutSTR functions.
    Also defined UART Interrupt & Handler
 */
#define WIFLY_UART 			LPC_UART0
#define WIFLY_BAUDRATE		9600

#define IRQ_WIFLY 			UART0_IRQn
#define HANDLER_WIFLY_IRQ 	UART0_IRQHandler
#define RING_BUFFER_SIZE	2048

/* Board name */
#define BOARD_NXP_LPCXPRESSO_1769


/**
 * LED RGB COLOR defines
 */
#define LED0_GPIO_PORT_NUM                      0
#define LED0_GPIO_BIT_NUM                       22
#define LED1_GPIO_PORT_NUM                      3
#define LED1_GPIO_BIT_NUM                       25
#define LED2_GPIO_PORT_NUM                      3
#define LED2_GPIO_BIT_NUM                       26

#define NUM_LEDS 			3
#define LED_RED           	0x00
#define LED_GREEN           0x01
#define LED_BLUE           	0x02
#define LED_ALL				0x03
static const uint8_t portLEDs[NUM_LEDS] = {
	LED0_GPIO_PORT_NUM,
	LED1_GPIO_PORT_NUM,
	LED2_GPIO_PORT_NUM
};
static const uint8_t pinLEDs[NUM_LEDS] = {
	LED0_GPIO_BIT_NUM,
	LED1_GPIO_BIT_NUM,
	LED2_GPIO_BIT_NUM
};
/* End LED RGB COLOR defines */

/*
 * IN signals defines
 */
#define IN_BRAKE_IN_GPIO_PORT_NUM				2
#define IN_BRAKE_IN_GPIO_BIT_NUM				0
#define IN_ACK_WARNING_GPIO_PORT_NUM			2
#define IN_ACK_WARNING_GPIO_BIT_NUM				1
#define IN_AUX1_GPIO_PORT_NUM					2
#define IN_AUX1_GPIO_BIT_NUM					2
#define IN_AUX2_GPIO_PORT_NUM					2
#define IN_AUX2_GPIO_BIT_NUM					3

#define NUM_IN_SIGNALS 4
#define BRAKE_IN			0		//binary position in status
#define ACK_WARNING			1		//binary position in status
#define DEBUG_IN			2		//binary position in status
#define WIFLY_JUMPER		3		//binary position in status
static const uint8_t portINSignal[NUM_IN_SIGNALS] = {
	IN_BRAKE_IN_GPIO_PORT_NUM,
	IN_ACK_WARNING_GPIO_PORT_NUM,
	IN_AUX1_GPIO_PORT_NUM,
	IN_AUX2_GPIO_PORT_NUM
};
static const uint8_t pinINSignal[NUM_IN_SIGNALS] = {
	IN_BRAKE_IN_GPIO_BIT_NUM,
	IN_ACK_WARNING_GPIO_BIT_NUM,
	IN_AUX1_GPIO_BIT_NUM,
	IN_AUX2_GPIO_BIT_NUM
};
static const uint8_t stateINSignal[NUM_IN_SIGNALS] = {
	BRAKE_IN,
	ACK_WARNING,
	DEBUG_IN,
	WIFLY_JUMPER
};
/* End IN signals defines */

/*
 * OUT signals defines
 */
#define OUT_WIFLY_RST_GPIO_PORT_NUM				0
#define OUT_WIFLY_RST_GPIO_BIT_NUM				21

#define NUM_OUT_SIGNALS 1
#define WIFLY_RST			0x01
static const uint8_t portOUTSignal[NUM_OUT_SIGNALS] = {
	OUT_WIFLY_RST_GPIO_PORT_NUM
};
static const uint8_t pinOUTSignal[NUM_OUT_SIGNALS] = {
	OUT_WIFLY_RST_GPIO_BIT_NUM
};
static const uint8_t stateOUTSignal[NUM_OUT_SIGNALS] = {
	WIFLY_RST
};
/* End OUT signals defines */


/**
 * @brief	Initialize pin muxing for a UART
 * @param	pUART	Pointer to UART register block for UART pins to init
 * @return	None
 */
void Board_UART_Init(LPC_USART_T *pUART);

/**
 * @brief	Returns the MAC address assigned to this board
 * @param	mcaddr	Pointer to 6-byte character array to populate with MAC address
 * @return	None
 * @note    Returns the MAC address used by Ethernet
 */
void Board_ENET_GetMacADDR(uint8_t *mcaddr);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __BOARD_H_ */
