/*
 * @brief Common board API functions
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 *
 * @note
 * modified Jordi Arroyo, 2016
 *
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 */

#ifndef __BOARD_API_H_
#define __BOARD_API_H_

#include "lpc_types.h"
#include "board.h"
#include "chip.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup BOARD_COMMON_API BOARD: Common board functions
 * @ingroup BOARD_Common
 * This file contains common board definitions that are shared across
 * boards and devices. All of these functions do not need to be
 * implemented for a specific board, but if they are implemented, they
 * should use this API standard.
 * @{
 */

/**
 * @brief	Setup and initialize hardware prior to call to main()
 * @return	None
 * @note	Board_SystemInit() is called prior to the application and sets up system
 * clocking, memory, and any resources needed prior to the application
 * starting.
 */
void Board_SystemInit(void);

/**
 * @brief	Setup pin multiplexer per board schematics
 * @return	None
 * @note	Board_SetupMuxing() should be called from SystemInit() prior to application
 * main() is called. So that the PINs are set in proper state.
 */
void Board_SetupMuxing(void);

/**
 * @brief	Setup system clocking 
 * @return	None
 * @note	This sets up board clocking.
 */
void Board_SetupClocking(void);

/**
 * @brief	Setup external system memory
 * @return	None
 * @note	This function is typically called after pin mux setup and clock setup and
 * sets up any external memory needed by the system (DRAM, SRAM, etc.). Not all
 * boards need this function.
 */
void Board_SetupExtMemory(void);

/**
 * @brief Function prototype for a MS delay function. Board layers or example code may
 *        define this function as needed.
 */
typedef void (*p_msDelay_func_t)(uint8_t, uint32_t);

/**
 * @brief	Set up and initialize all required blocks and functions related to the board hardware.
 * @return	None
 */
void Board_Init(void);

/**
 * @brief	Initializes board DEBUG UART for output, required for DEBUG communication
 * @return	None
 */
void Board_Debug_Init(void);

/**
 * @brief	Sends a single character on the DEBUG UART, required for DEBUG communication
 * @param	ch	character to send
 * @return	None
 */
void Board_UARTPutChar(char ch);

/**
 * @brief	Get a single character from the DEBUG UART, required for DEBUG input
 * @return	EOF if not character was received, or character value
 */
int Board_UARTGetChar(void);

/**
 * @brief	Send a string to the DEBUG UART
 * @param	str	Terminated string to output
 * @return	None
 */
void Board_UARTPutSTR(char *str);

/**
 * @brief	Initializes board UART for WIFLY, required for wifly communication
 * @return	None
 */
void Uart_Wifly_Init(void);

/**
 * @brief	Sends a single character on the WIFLY UART, required for WIFLY communication
 * @param	ch	character to send
 * @return	None
 */
void Wifly_UARTPutChar( char ch );

/**
 * @brief	Get a single character from the WIFLY UART, required for WIFLY input
 * @return	EOF if not character was received, or character value
 */
int Wifly_UARTGetChar( void );

/**
 * @brief	Send a string to the WIFLY UART
 * @param	str	Terminated string to output
 * @return	None
 */
void Wifly_UARTPutSTR( char *str );

/**
 * @brief	Read for the Ring Buffer of the WIFLY UART
 * 			for a max Times, with waitedString detected reduce Time
 * @param	pcString : pointer to put the read String
 * @param	waitedString : pattern to compare
 * @param	pDelayMsFunc : pointer to function Delay
 * @param	Time : maxim time to wait
 * @return	None
 */
void readWiiflyRingBuffer(const char *pcString, const char *waitedString, p_msDelay_func_t pDelayMsFunc, uint32_t Time);

/**
 * @brief	Sets the state of a board LED to on or off
 * @param	LEDNumber	LED number to set state for
 * 						  0-Red, 1-Green, 2-Blue, 3-ALL
 * @param	State		true for on, false for off
 * @return	None
 */
void Board_LED_Set(uint8_t LEDNumber, bool State);

/**
 * @brief	Returns the current state of a board LED
 * @param	LEDNumber	LED number to set state for
 * 						  0-Red, 1-Green, 2-Blue, 3-ALL
 * @return	true if the LED is on, otherwise false
 */
bool Board_LED_Test(uint8_t LEDNumber);

/**
 * @brief	Toggles the current state of a board LED
 * @param	LEDNumber	LED number to change state for
 * 						  0-Red, 1-Green, 2-Blue, 3-ALL
 * @return	None
 */
void Board_LED_Toggle(uint8_t LEDNumber);

/**
 * @brief	Get Global INs Signals status
 * @return	status of IN signals
 * 			binary determine active or not
 * 			0 active, 1 non active
 */
uint8_t IN_GetStatus(void);

/**
 * @brief	Get Individual IN Signal status
 * @param   in_signal	Signal to get status
 * @return	status of individual IN signals
 * 			true active, false not active
 */
bool IN_GetStatusSignal(uint8_t IN_signal);

/**
 * @brief	Set OUTs Signals status
 * @param	OUT	Out signal to set status
 * @param	On	Status to set true active, false not active
 * @return	None
 */
void OUT_Set( uint8_t OUT, bool on);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __BOARD_API_H_ */
