/*
 * @brief NXP LPC1769 LPCXpresso board file
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 *
 * @note
 * modified Jordi Arroyo, 2016
 *
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 */

#include "board.h"
#include "string.h"

#include "retarget.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* Receive Wifly ring buffer */
static RINGBUFF_T rxring;

/* Transmit and receive buffers */
static uint8_t rxbuff[RING_BUFFER_SIZE];

/* System oscillator rate and RTC oscillator rate */
const uint32_t OscRateIn = 12000000;
const uint32_t RTCOscRateIn = 32768;

/* Pointer to delay function */
static p_msDelay_func_t pDelayMs;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Initializes board LED(s) */
static void Board_LED_Init(void)
{
	/* Pin P0.22 is configured as GPIO pin during SystemInit */
	/* Pin P3.25 is configured as GPIO pin during SystemInit */
	/* Pin P3.26 is configured as GPIO pin during SystemInit */

	int ix;

	/* Set the P0.22, P3.25 &  P3.26 as output */
	for (ix = 0; ix < NUM_LEDS; ix++) {
		Chip_GPIO_SetPinDIROutput(LPC_GPIO, portLEDs[ix], pinLEDs[ix]);
	}
	/* ALL color off */
	Board_LED_Set(LED_ALL, false);
}

/* Initialize INs_OUTs*/
void Board_IN_OUT_Init(void)
{
	int ix;

	/* IOCON states already selected in SystemInit(), GPIO setup only. */

	// INs
	for (ix = 0; ix < NUM_IN_SIGNALS; ix++) {
		Chip_GPIO_SetPinDIRInput(LPC_GPIO, portINSignal[ix], pinINSignal[ix]);
	}

	// OUTs
	for (ix = 0; ix < NUM_OUT_SIGNALS; ix++) {
		Chip_GPIO_SetPinDIROutput(LPC_GPIO, portOUTSignal[ix], pinOUTSignal[ix]);
	}
	/* force up WIFLY_RST to avoid reset of WIFLY board*/
	OUT_Set( WIFLY_RST, true);

}



/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* UART WIFLY interrupt handler using ring buffers */
void HANDLER_WIFLY_IRQ(void)
{
	/* Use default ring buffer handler */
	Chip_UART_RXIntHandlerRB(WIFLY_UART, &rxring);
}

/* Initialize UART pins */
void Board_UART_Init(LPC_USART_T *pUART)
{
	/* Pin Muxing has already been done during SystemInit */
}

/* Set up and initialize all required blocks and functions related to the
   board hardware */
void Board_Init(void)
{
	/* Sets up DEBUG UART */
	Board_Debug_Init();

	/* Sets up WIFLY UART */
	Uart_Wifly_Init();

	/* Initializes GPIO */
	Chip_GPIO_Init(LPC_GPIO);
	Chip_IOCON_Init(LPC_IOCON);

	/* Initialize LEDs */
	Board_LED_Init();

	/* Initialize IN_OUT Signals */
	Board_IN_OUT_Init();
}

/* Initialize debug output via UART for board */
void Board_Debug_Init(void)
{
	Board_UART_Init(DEBUG_UART);

	Chip_UART_Init(DEBUG_UART);
	Chip_UART_SetBaud(DEBUG_UART, DEBUG_BAUDRATE);
	Chip_UART_ConfigData(DEBUG_UART, UART_LCR_WLEN8 | UART_LCR_SBS_1BIT | UART_LCR_PARITY_DIS);

	/* Enable UART Transmit */
	Chip_UART_TXEnable(DEBUG_UART);
}

/* Sends a character on the UART */
void Board_UARTPutChar(char ch)
{
	while ((Chip_UART_ReadLineStatus(DEBUG_UART) & UART_LSR_THRE) == 0) {}
	Chip_UART_SendByte(DEBUG_UART, (uint8_t) ch);
}

/* Gets a character from the UART, returns EOF if no character is ready */
int Board_UARTGetChar(void)
{
	if (Chip_UART_ReadLineStatus(DEBUG_UART) & UART_LSR_RDR) {
		return (int) Chip_UART_ReadByte(DEBUG_UART);
	}
	return EOF;
}

/* Outputs a string on the debug UART */
void Board_UARTPutSTR(char *str)
{
	while (*str != '\0') {
		Board_UARTPutChar(*str++);
	}
}

/* Initialize UART board for wifly communication */
void Uart_Wifly_Init(void)
{
	Board_UART_Init(WIFLY_UART);

	Chip_UART_Init(WIFLY_UART);
	Chip_UART_SetBaud(WIFLY_UART, WIFLY_BAUDRATE);
	Chip_UART_ConfigData(WIFLY_UART, UART_LCR_WLEN8 | UART_LCR_SBS_1BIT | UART_LCR_PARITY_DIS);

	/* Initialize the ring buffer */
	RingBuffer_Init(&rxring, rxbuff, 1, RING_BUFFER_SIZE);

	/* Reset and enable FIFO (in this case only FIFO Rx) FIFO trigger level 3 (14 chars) */
	Chip_UART_SetupFIFOS(WIFLY_UART, (UART_FCR_FIFO_EN | UART_FCR_RX_RS | UART_FCR_TRG_LEV3));

	/* Enable receive data and line status interrupt */
	Chip_UART_IntEnable(WIFLY_UART, (UART_IER_RBRINT | UART_IER_RLSINT));

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(IRQ_WIFLY, 1);
	NVIC_EnableIRQ(IRQ_WIFLY);

	/* Enable UART Transmit */
	Chip_UART_TXEnable(WIFLY_UART);
}

/* Sends a character on the parameter UART */
void Wifly_UARTPutChar( char ch )
{
	while ((Chip_UART_ReadLineStatus(WIFLY_UART) & UART_LSR_THRE) == 0) {}
	Chip_UART_SendByte(WIFLY_UART, (uint8_t) ch);
}

/* Gets a character from the parameter UART, returns EOF if no character is ready */
int Wifly_UARTGetChar( void )
{
	if (Chip_UART_ReadLineStatus(WIFLY_UART) & UART_LSR_RDR) {
		return (int) Chip_UART_ReadByte(WIFLY_UART);
	}
	return EOF;
}

/* Outputs a string on the parameter UART */
void Wifly_UARTPutSTR( char *str )
{
	while (*str != '\0') {
		Wifly_UARTPutChar(*str++);
	}
}

/* Read for the Ring Buffer of the WIFLY UART */
void readWiiflyRingBuffer(const char *pcString, const char *waitedString, p_msDelay_func_t pDelayMsFunc, uint32_t Time)
{
	pDelayMs = pDelayMsFunc;
	bool done = false;

	while (Time > 0){
		Chip_UART_ReadRB(WIFLY_UART, &rxring, rxbuff, RING_BUFFER_SIZE);
		if (rxbuff[0] != '\0') {
			strcat((char *)pcString, (char *)rxbuff);
			memset((uint8_t *)rxbuff, 0x00, RING_BUFFER_SIZE);
		}
		if ( (bool)strstr (pcString, waitedString) && !done ) {
			Time = 10;
			done = true;
		}
		pDelayMs(0,1);
		Time--;
	}
}

/* Sets the state of a board LED to on or off */
void Board_LED_Set(uint8_t LEDNumber, bool On)
{
	/* In Rev.D RGB LED turn on with low level */
	 if (LEDNumber == LED_ALL)
	{
		int ix;
		for (ix = 0; ix < NUM_LEDS; ix++) {
			Chip_GPIO_SetPinState(LPC_GPIO, portLEDs[ix], pinLEDs[ix], !On);
		}
	}
	else
		Chip_GPIO_SetPinState(LPC_GPIO, portLEDs[LEDNumber], pinLEDs[LEDNumber], !On);
}

/* Returns the current state of a board LED */
bool Board_LED_Test(uint8_t LEDNumber)
{
	/* In Rev.D RGB LED turn on with low level */
	return Chip_GPIO_GetPinState(LPC_GPIO, portLEDs[LEDNumber], pinLEDs[LEDNumber]);
}

/* Toggle a Board LED */
void Board_LED_Toggle(uint8_t LEDNumber)
{
	/* In Rev.D RGB LED turn on with low level */
	Board_LED_Set(LEDNumber, Board_LED_Test(LEDNumber));
}

/* Get Global INs Signals status */
uint8_t IN_GetStatus(void)
{
	uint8_t ix, ret = 0;

	for (ix = 0; ix < NUM_IN_SIGNALS; ix++) {
		if ((Chip_GPIO_GetPinState(LPC_GPIO, portINSignal[ix], pinINSignal[ix])) == true) {
			ret |= (1 << stateINSignal[ix]);
		}
	}
	return ret;
}

/* Get Individual IN Signal status */
bool IN_GetStatusSignal(uint8_t IN_signal)
{
	return (bool) Chip_GPIO_GetPinState(LPC_GPIO, portINSignal[IN_signal], pinINSignal[IN_signal]);
}

/* Sets state of OUT to on or off */
void OUT_Set( uint8_t OUT, bool on)
{
	Chip_GPIO_SetPinState(LPC_GPIO, portOUTSignal[OUT], pinOUTSignal[OUT], on);
}

/* Returns the MAC address assigned to this board */
void Board_ENET_GetMacADDR(uint8_t *mcaddr)
{
	const uint8_t boardmac[] = {0x00, 0x60, 0x37, 0x12, 0x34, 0x56};

	memcpy(mcaddr, boardmac, 6);
}
