/*
 * @brief ADC handler
 * This handler is for the ADC in Polling mode for ADD0, ADD1 & ADD2
 *
 * @note
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 *
 * Jordi Arroyo, 2016
 */

#include "adc.h"
#include "board.h"
#include "aux_print.h"

#include "FreeRTOS.h"
#include "task.h"

#include <math.h>


/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/
static ADC_CLOCK_SETUP_T ADCSetup;

static uint16_t X_offset, Y_offset, Z_offset;
static uint16_t dataADC_X, dataADC_Y, dataADC_Z;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*  Calculation Brake function & print in DEVELOPMENT_ADC_MODE */
static void Brake_ADC_valueXYZ(float *acceleration)
{
#ifdef ACCURACY_FLOAT
	float sensitivity;		// 800mv/g
	float VDD;				// V Reference
	float rateADC;			// rate of ADC conversion
	float Vx, Vy, Vz;		// V Out MMA7361
	float G, g, Gx, Gy, Gz;	// G-module & 3-axes

	sensitivity = 0.8;
	VDD = 3.3;
	rateADC = 4095;	// 12 bits
	g = 9.81;						// Constant of g 10 for simplification

	Vx = (dataADC_X - X_offset) * VDD / rateADC;
	Vy = (dataADC_Y - Y_offset) * VDD / rateADC;
	Vz = (dataADC_Z - Z_offset) * VDD / rateADC;

	Gx = Vx / sensitivity;
	Gy = Vy / sensitivity;
	Gz = ( Vz / sensitivity ) + 1;

	/* calculation of global module of G */
	G = sqrt( pow(Gx,2) + pow(Gy,2) + pow(Gz,2));

	/* Assuming 1g in axis z acceleration is that made change module of G */
	*acceleration = g * sqrt( fabs(pow(G,2) - 1) );

	#if defined(DEVELOPMENT_ADC_MODE)
		char buf[DEBUG_SIZE];
		vPrintString( "ADC value is :" );
		sprintf(buf,"X:   %d  %.4lfV  %.4fg", dataADC_X, Vx, Gx);
		vPrintString( buf );
		sprintf(buf,"Y:   %d  %.4lfV  %.4fg", dataADC_Y, Vy, Gy);
		vPrintString( buf );
		sprintf(buf,"Z:   %d  %.4lfV  %.4fg", dataADC_Z, Vz, Gz);
		vPrintString( buf );
		sprintf(buf,"MODULE OF G: %.4f", G);
		vPrintString( buf );
		sprintf(buf,"MODULE OF A_BRAKE: %.4f", &acceleration);
		vPrintString( buf );
	#endif /* defined(DEVELOPMENT_ADC_ENABLE)*/

#else


	/* Nine bits of accuracy
	 * to avoid decimals in Voltage use mv in VDD reference
	 * to avoid decimals in G use sensitivity/100 & g defined 10
	 * Result of G, Gx, Gy, Gz has 2 implicit decimals
	 * */

	uint16_t sensitivity;		// 800mV/g
	uint16_t VDD;				// mV Reference
	uint16_t rateADC;			// rate of ADC conversion
	uint16_t Vx, Vy, Vz;		// V Out MMA7361
	uint16_t G, g, Gx, Gy, Gz;	// G-module & 3-axes

	VDD = 3300;
	rateADC = 511;				// 9 bits
	sensitivity = 8;
	g = 10;						// Constant of g 10 for simplification

	#if defined(ACCURACY_9_BITS)
		dataADC_X = dataADC_X >> 3;
		dataADC_Y = dataADC_Y >> 3;
		dataADC_Z = dataADC_Z >> 3;
	#endif /* defined (ACCURACY_9_BITS) */

	/* assumes train only go forward */
	if (dataADC_X < X_offset)
		dataADC_X = X_offset;
	if (dataADC_Y < Y_offset)
		dataADC_Y = Y_offset;
	if (dataADC_Z < Z_offset)
		dataADC_Z = Z_offset;

	Vx = (dataADC_X - X_offset) * VDD / rateADC;
	Vy = (dataADC_Y - Y_offset) * VDD / rateADC;
	Vz = (dataADC_Z - Z_offset) * VDD / rateADC;

	Gx = ( Vx / sensitivity );
	Gy = ( Vy / sensitivity );
	Gz = ( Vz / sensitivity ) + 1;		// assumes the board is plane

	/* calculation of global of G */
	G = sqrt( pow(Gx,2) + pow(Gy,2) + pow(Gz,2));

	/* Assuming 1g in axis z acceleration is that made change module of G */
	*acceleration = (g * sqrt( (pow(G,2) - 1) ))/100;

	#if defined(DEVELOPMENT_ADC_MODE)
	char buf[DEBUG_SIZE];
		vPrintString( "ADC value is :" );
		sprintf(buf,"X:   %d  %dmV  %dg*10^-2", dataADC_X, Vx, Gx);
		vPrintString( buf );
		sprintf(buf,"Y:   %d  %dmV  %dg*10^-2", dataADC_Y, Vy, Gy);
		vPrintString( buf );
		sprintf(buf,"Z:   %d  %dmV  %dg*10^-2", dataADC_Z, Vz, Gz);
		vPrintString( buf );
		sprintf(buf,"MODULE OF G: %dg*10^-2", G);
		vPrintString( buf );
		sprintf(buf,"MODULE OF A_BRAKE: %.4f", &acceleration);
		vPrintString( buf );
	#endif /* defined(DEVELOPMENT_ADC_ENABLE) */

#endif /* defined ACCURACY_FLOAT */
}


/* Read ADC Values in polling mode */
static void App_Polling_Test(void)
{
	/* Waiting for ADD0 A/D conversion complete */
	while (Chip_ADC_ReadStatus(_LPC_ADC_ID, _ADC_CHANNLE_X, ADC_DR_DONE_STAT) != SET) {}
	/* Read ADD0 ADC value */
	Chip_ADC_ReadValue(_LPC_ADC_ID, _ADC_CHANNLE_X, &dataADC_X);

	/* Waiting for ADD1 A/D conversion complete */
	while (Chip_ADC_ReadStatus(_LPC_ADC_ID, _ADC_CHANNLE_Y, ADC_DR_DONE_STAT) != SET) {}
	/* Read ADD1 ADC value */
	Chip_ADC_ReadValue(_LPC_ADC_ID, _ADC_CHANNLE_Y, &dataADC_Y);

	/* Waiting for ADD2 A/D conversion complete */
	while (Chip_ADC_ReadStatus(_LPC_ADC_ID, _ADC_CHANNLE_Z, ADC_DR_DONE_STAT) != SET) {}
	/* Read ADD2 ADC value */
	Chip_ADC_ReadValue(_LPC_ADC_ID, _ADC_CHANNLE_Z, &dataADC_Z);
}

/* Calculation of offset values */
static void offsetADC(void)
{
	int rate = 0;

	#if defined(DEVELOPMENT_ADC_MODE)
	char buf[DEBUG_SIZE];
		vPrintString( "Determining Offset ..." );
	#endif /* defined(DEVELOPMENT_ADC_ENABLE) */

	// Delay for stabilization of reading
	Delay_Ms(EXEC_STATUS, 1000);

	while (rate < 10000)//0xfffff)
	{
		App_Polling_Test();
		// Recording value for first test as offset
		if (rate == 0) {
			X_offset = dataADC_X;
			Y_offset = dataADC_Y;
			Z_offset = dataADC_Z;
		}
		// Recording average as offset
		else {
			X_offset = (X_offset + dataADC_X)/2;
			Y_offset = (Y_offset + dataADC_Y)/2;
			Z_offset = (Z_offset + dataADC_Z)/2;
		}
		rate++;
	}
	#if defined(ACCURACY_9_BITS)
		X_offset =  X_offset >> 3;
		Y_offset =  Y_offset >> 3;
		Z_offset =  Z_offset >> 3;
	#endif /* defined (ACCURACY_9_BITS) */

	#if defined(DEVELOPMENT_ADC_MODE)
		vPrintString( "Offset value is :" );
		sprintf(buf,"Offset(x: %d, y: %d, z: %d)", X_offset, Y_offset, Z_offset);
		vPrintString( buf );
	#endif /* defined(DEVELOPMENT_ADC_ENABLE) */
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Initializes board ADC & channels enable */
void adc_Init(void) {
	/* Pin muxing configuration  in lpc_board_nxp_nxp_lpcxpresso_1769/board_sysnt.c */

	/*ADC Init */
	Chip_ADC_Init(_LPC_ADC_ID, &ADCSetup);

	/* Enable channels */
	Chip_ADC_EnableChannel(_LPC_ADC_ID, _ADC_CHANNLE_X, ENABLE);
	Chip_ADC_EnableChannel(_LPC_ADC_ID, _ADC_CHANNLE_Y, ENABLE);
	Chip_ADC_EnableChannel(_LPC_ADC_ID, _ADC_CHANNLE_Z, ENABLE);

	/* Enable Burst Mode Enable */
	Chip_ADC_SetBurstCmd(_LPC_ADC_ID, ENABLE);

	// Determining INI values assuming position Gx=0, Gx=0 & Gz=1
	offsetADC();
}

/* read & print in DEBUG_MODE ADC VAlues */
extern void readADC(float *acc_brake) {
	vTaskSuspendAll();
	{
		App_Polling_Test();
		Brake_ADC_valueXYZ(acc_brake);
	}
	xTaskResumeAll();
}







