/*
 * @brief aux_io.h
 * Auxiliary functions for I/O UARTs
 *
 * @note
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 *
 * Jordi Arroyo, 2016
 */

#include "aux_print.h"
#include "board.h"
#include "types.h"

#include <stdio.h>
#include <string.h>

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"


/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

static char dBuffer[ DEBUG_SIZE ];
static char wBuffer[ WIFLY_SIZE ];
static char rbBuffer[ RING_BUFFER_SIZE ];

static bool DEBUG_MODE = false;

/* Declare a variable of type xSemaphoreHandle */
volatile static xSemaphoreHandle pMutex = NULL;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/


/*****************************************************************************
 * Private functions
 ****************************************************************************/


/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Set DEBUG_MODE on or off */
void setDebugMode( bool on )
{
	DEBUG_MODE = on;
}

/*
 * Send to DEBUG UART the string,
 * suspending the scheduler as method of mutual exclusion.
*/
void vPrintString( const char *pcString )
{
	vTaskSuspendAll();
	{
		sprintf( dBuffer, "%s %s", pcString , ENDRETURN);
		if (DEBUG_MODE)
			Board_UARTPutSTR(dBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to DEBUG UART the string without return end,
 * suspending the scheduler as method of mutual exclusion.
*/
void vPrintStringNoReturn( const char *pcString )
{
	vTaskSuspendAll();
	{
		sprintf( dBuffer, "%s", pcString );
		if (DEBUG_MODE)
			Board_UARTPutSTR(dBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to DEBUG UART the strings concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vPrintStringAndString( const char *pcString1, const char *pcString2 )
{
	vTaskSuspendAll();
	{
		sprintf( dBuffer, "%s %s%s", pcString1 , pcString2, ENDRETURN);
		if (DEBUG_MODE)
			Board_UARTPutSTR(dBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to DEBUG UART the string & Integer number concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vPrintStringAndNumber( const char *pcString, unsigned long ulValue )
{
	vTaskSuspendAll();
	{
		sprintf( dBuffer, "%s %lu %s", pcString, ulValue , ENDRETURN);
		if (DEBUG_MODE)
			Board_UARTPutSTR(dBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to DEBUG UART the string & Decimal number concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vPrintStringAndDecimal( const char *pcString, double udValue )
{
	vTaskSuspendAll();
	{
		sprintf( dBuffer, "%s %.2f %s", pcString, udValue , ENDRETURN);
		if (DEBUG_MODE)
			Board_UARTPutSTR(dBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to WIFLY UART the string,
 * suspending the scheduler as method of mutual exclusion.
*/
void vSendWiflyString( const portCHAR *pcString )
{
	vTaskSuspendAll();
	{
		sprintf( wBuffer, "%s%s", pcString , ENDRETURN);
		Wifly_UARTPutSTR( wBuffer );
	}
	xTaskResumeAll();
}

/*
 * Send to WIFLY UART the string without return end,
 * suspending the scheduler as method of mutual exclusion.
*/
void vSendWiflyStringNoReturn( const portCHAR *pcString )
{
	vTaskSuspendAll();
	{
		Wifly_UARTPutSTR( (char *)pcString );
	}
	xTaskResumeAll();
}

/*
 * Send to WIFLY UART the strings concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vSendWiflyStringAndString( const portCHAR *pcString1, const portCHAR *pcString2 )
{
	vTaskSuspendAll();
	{
		sprintf( wBuffer, "%s %s%s", pcString1 , pcString2 , ENDRETURN);
		Wifly_UARTPutSTR( wBuffer );
	}
	xTaskResumeAll();
}

/*
 * Send to WIFLY UART the string & Integer number concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vSendWiflyStringAndNumber( const char *pcString, unsigned long ulValue )
{
	vTaskSuspendAll();
	{
		sprintf( wBuffer, "%s %lu%s", pcString, ulValue , ENDRETURN);
		Wifly_UARTPutSTR( wBuffer);
	}
	xTaskResumeAll();
}

/*
 * Send to WIFLY UART the string & Decimal number concatenation
 * suspending the scheduler as method of mutual exclusion.
*/
void vSendWiflyStringAndDecimal( const char *pcString, double udValue )
{
	vTaskSuspendAll();
	{
		sprintf( wBuffer, "%s %.2f%s", pcString, udValue , ENDRETURN);
		Wifly_UARTPutSTR( wBuffer);
	}
	xTaskResumeAll();
}

/* Check the wifly response, & return true is done */
bool checkWiflyResponse( const char *waitedString, uint32_t Time )
{
	bool done = false;
	memset((char *)rbBuffer,0,sizeof(rbBuffer));

	readWiiflyRingBuffer( rbBuffer, waitedString, Delay_Ms, Time);
	done = (bool)strstr (rbBuffer, waitedString);
	if ( done )
		vPrintStringAndString( (char *) "------- RESPONSE OK -------\r\n", rbBuffer );
	else
		vPrintStringAndString( (char *) "----- MISSED RESPONSE -----\r\n", rbBuffer );

	return done;
}

/* return String with System State */
void getStringStatusSystem(const char *pcString, SYSTEM_STATUS_T state )
{
	memset((char *)pcString,0,sizeof(pcString));
	switch (state){
	case SYSTEM_ON:
		sprintf( (char *)pcString, "SYSTEM ON");
		break;
	case SYSTEM_BRAKE_ACT:
		sprintf( (char *)pcString, "BRAKE ACTIVATION");
		break;
	case SYSTEM_WARNING:
		sprintf( (char *)pcString, "WARNING");
		break;
	case SYSTEM_EMERGENCY_BRAKE:
		sprintf( (char *)pcString, "EMERGENCY BRAKE");
		break;
	default:	/* Should never arrive here */
		sprintf( (char *)pcString, "OUT OF STATE");
		break;
	}
}

/* return String with active INs to the System */
void getStringINs(const char *pcString, uint8_t statusIN)
{
	memset((char *)pcString,0,sizeof(pcString));
	if (statusIN == 0) {
		sprintf( (char *)pcString, "%s%s", pcString , "NOT_IN_ACTIVE ");
	}
	else
	{
		if ((bool)(statusIN & (1 << BRAKE_IN)))
			sprintf( (char *)pcString, "%s%s", pcString , "BRAKE_IN ");
		if ((bool)(statusIN & (1 << ACK_WARNING)))
			sprintf( (char *)pcString, "%s%s", pcString , "ACK_WARNING ");
		if ((bool)(statusIN & (1 << DEBUG_IN)))
			sprintf( (char *)pcString, "%s%s", pcString , "DEBUG_IN ");
		if ((bool)(statusIN & (1 << WIFLY_JUMPER)))
			sprintf( (char *)pcString, "%s%s", pcString , "WIFLY_JUMPER ");
	}
}

/**
 * @brief	Wait time in execution:
 * @param	delayMOde(Sequential or Task), ms milliseconds to delay
 * @return	None
 * @note    Blocking for sequential & vTaskDelayUntil(constant time) in SO
 */
void Delay_Ms(uint8_t delayMode, uint32_t ms)
{
	// Initialize the xLastWakeTime variable with the current time.
	portTickType xLastWakeTime;
	xLastWakeTime = xTaskGetTickCount();

	if (delayMode == EXEC_SECC){
		/* Sequential execution */
		uint32_t i;
		for(i=0; i<(((uint32_t)6850)*ms);i++); // 6850 loops ~ 1 millisecond
	}
	else {
		/* Task execution */
		vTaskDelayUntil( &xLastWakeTime, (portTickType) ms );
	}
}
