/*
 * @brief main
 *
 * @note
 * UNIVERSITAT OBERTA DE CATALUNYA
 * TREBALL FINAL DE CARRERA - SISTEMES ENCASTATS
 *
 * Jordi Arroyo, 2016
 */

#include "board.h"
#include "adc.h"
#include "aux_print.h"
#include "string.h"
#include "types.h"
#include "wifly.h"
#include "iniConfig.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"
#include "timers.h"


/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* The number of items the queue can hold.  This is 1 as the receive task
has a higher priority than the send task, so will remove items as they are added,
meaning the send task should always find the queue empty. */
#define mainQUEUE_LENGTH					( 1 )

/* The tasks of the system */
static void vWIFLYTask(void *pvParameters);
static void vDEBUGTask(void *pvParameters);
static void vADCTask(void *pvParameters);
static void vINTask(void *pvParameters) ;
static void vOUTTask(void *pvParameters);
static void vMAINTask( void *pvParameters );

/* The queue used by Wifly & Debug task. */
static xQueueHandle xQueue = NULL;

/* Declare a variable of type xSemaphoreHandle.  This is used to reference the
mutex type semaphore that is used to ensure mutual exclusive access to stdout. */
static xSemaphoreHandle xMutex = NULL;

/* The callback function assigned to Reaction timer */
static void vReactionTimerCallback( xTimerHandle xTimer );

/* The callback function assigned to Deteccion timer */
static void vDeteccionTimerCallback( xTimerHandle xTimer );

/* Timer to control Reaction Time for Brake */
static xTimerHandle xReactionTimer = NULL;

/* Timer to control Deteccion Time for Brake */
static xTimerHandle xDeteccionTimer = NULL;

/* Structure that contains all the functional data of the system */
static STATUS_GLOBAL data;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Sets up system hardware */
static void prvSetupHardware(void)
{
	/* Initial mode of work register for Delays */
	EXEC_STATUS = EXEC_SECC;

	SystemCoreClockUpdate();
	Board_Init();

	/* Read IN Signals to determinate DEBUG_MODE */
	setDebugMode( IN_GetStatusSignal(DEBUG_IN) );

	/* Read IN signals & in case of WIFLY_JUMPER active
	 * configure board wifly for UDP communication
	 */
	while (IN_GetStatusSignal(WIFLY_JUMPER)) {
		/* reset wifly board */
		resetWifly();
		if (wifly_Configuration()) {
			vPrintString( (char *)"... Remove Jumper");
			while (IN_GetStatusSignal(WIFLY_JUMPER)) {
				Board_LED_Toggle(LED_BLUE);
				Delay_Ms( EXEC_STATUS, 100);
			}
		}
	}

	/* test LEDs */
	for (int color = 0; color < NUM_LEDS; color++)
	{
		Board_LED_Set(color, true);
		Delay_Ms( EXEC_STATUS, 1000);
		Board_LED_Set(LED_ALL, false);
		Delay_Ms( EXEC_STATUS, 100);
	}

	/* Init ADC */
	adc_Init();

	/* Join Wifly board to Wi-fi net, reset board if error occurs */
	while ( !wifly_Init())
	{
		/* Reset Wifly Board if error */
		resetWifly();
	}

	/* Net test Only for development - Delete comment to do */
	// pingTest();
	// showIP();

	/* Board in State SYSTEM_ON */
	Board_LED_Set(LED_GREEN, true);

	/* Initialing structure data */
	readADC(&data.a_value);
	data.statusIN = IN_GetStatus();
	data.mode = SYSTEM_ON;

	/* Change mode of work register  for Delays */
	EXEC_STATUS = EXEC_TASK;
}


/* DEBUG PUBLICATOR TASK */
static void vDEBUGTask(void *pvParameters) {
	static char cBuffer[ WIFLY_SIZE ];
	static char sBuffer[ DEBUG_SIZE ];
	static char iBuffer[ DEBUG_SIZE ];
	static char auxBuffer[ AUX_SIZE ];

	while (1)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/* Create the String with the state of the System
			 * In DEBUG_MODE send to the DEBUF UART
			 * Always send to the wifly task by the queue
			 */
			memset((char *)cBuffer,0,sizeof(cBuffer));
			memset((char *)sBuffer,0,sizeof(sBuffer));
			memset((char *)iBuffer,0,sizeof(iBuffer));
			memset((char *)auxBuffer,0,sizeof(auxBuffer));

			sprintf(cBuffer, (char *)"--- Acceleration : %.2f m/s^2%s", data.a_value, ENDRETURN );
			vPrintStringNoReturn(cBuffer);

			getStringStatusSystem(auxBuffer, data.mode  );
			sprintf(sBuffer, (char *)"    System State : %s%s", auxBuffer,  ENDRETURN );
			vPrintStringNoReturn(sBuffer);

			getStringINs(auxBuffer, data.statusIN);
			sprintf(iBuffer, (char *)"    INs Status   : %s%s", auxBuffer,  ENDRETURN );
			vPrintString(iBuffer);

			/* create buffer to send to the queue */
			sprintf(cBuffer,"%s%s%s", cBuffer, sBuffer, iBuffer);
		}
		xSemaphoreGive( xMutex );

		/* Send to the queue the Buffer with the String to transmit */
		xQueueSend( xQueue, &cBuffer, 0 );

		/* About a 500ms delay here */
		Delay_Ms(EXEC_STATUS, TIME_MSG);
	}
}

/* WIFLY TASK */
static void vWIFLYTask(void *pvParameters)
{
	static char cBuffer[ WIFLY_SIZE ];
	while (1)
	{
		/* Wait until something arrives in the queue */
		xQueueReceive( xQueue, &cBuffer, portMAX_DELAY);

		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			vSendWiflyString(cBuffer);
		}
		xSemaphoreGive( xMutex );
	}
}

/*  ADC TASK */
static void vADCTask(void *pvParameters) {
	while (1) {
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/* Only for development */
			//vPrintString( (char *)"... Task ADC" );

			readADC(&data.a_value );
		}
		xSemaphoreGive( xMutex );

		/* About a 10ms delay here */
		Delay_Ms(EXEC_STATUS, TIME_TASK);
	}
}

/*  IN Handler TASK */
static void vINTask(void *pvParameters) {
	while (1) {
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/* Only for development */
			//vPrintString( (char *)"... Task IN" );

			/* Read State of IN signals to store */
			data.statusIN = IN_GetStatus();

			/* check DEBUG_IN Signal to determinate DEBUG_MODE */
			setDebugMode( IN_GetStatusSignal(DEBUG_IN) );
		}
		xSemaphoreGive( xMutex );

		/* About a 10ms delay here */
		Delay_Ms(EXEC_STATUS, TIME_TASK);
	}
}

/*  OUT Handler TASK */
static void vOUTTask(void *pvParameters) {
	while (1) {
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/* Only for development */
			//vPrintString( (char *)"... Task OUT" );

			switch(data.mode){
			case SYSTEM_ON:
				Board_LED_Set(LED_ALL, false);
				Board_LED_Set(LED_GREEN, true);
				break;
			case SYSTEM_BRAKE_ACT:
				Board_LED_Set(LED_BLUE, false);
				Board_LED_Toggle(LED_GREEN);
				break;
			case SYSTEM_WARNING:
				Board_LED_Set(LED_ALL, false);
				Board_LED_Set(LED_BLUE, true);
				break;
			case SYSTEM_EMERGENCY_BRAKE:
				Board_LED_Set(LED_ALL, false);
				Board_LED_Set(LED_RED, true);
				break;
			}
		}
		xSemaphoreGive( xMutex );

		/* About a 10ms delay here */
		Delay_Ms(EXEC_STATUS, TIME_TASK);
	}
}

/*  States reviser TASK */
static void vMAINTask(void *pvParameters) {
	while (1) {
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			/* Only for development */
			//vPrintString( (char *)"... Task MAIN" );

			switch(data.mode){
			case SYSTEM_ON:
				if (IN_GetStatusSignal(BRAKE_IN))
				{
					/* Activate Timers & change state */
					if (!xTimerIsTimerActive(xDeteccionTimer)) {
						xTimerStart( xDeteccionTimer, 0 );
					}
					if (!xTimerIsTimerActive(xReactionTimer)) {
						xTimerStart( xReactionTimer, 0 );
					}
					data.mode = SYSTEM_BRAKE_ACT;				}
				break;

			case SYSTEM_BRAKE_ACT:
				if (!IN_GetStatusSignal(BRAKE_IN)) {
					/* Deactivate Timers & change state */
					if (xTimerIsTimerActive(xDeteccionTimer)) {
						xTimerStop( xDeteccionTimer, 0 );
					}
					if (xTimerIsTimerActive(xReactionTimer)) {
						xTimerStop( xReactionTimer, 0 );
					}
					data.mode = SYSTEM_ON;
				}
				else if (data.a_value < BRAKEMIN && !xTimerIsTimerActive(xDeteccionTimer) ) {
					/* If timer of deteccion is expired go warning */
					data.mode = SYSTEM_WARNING;
				}
				break;

			case SYSTEM_WARNING:
				if (!IN_GetStatusSignal(BRAKE_IN)) {
					/* Deactivate Timers & change state */
					if (xTimerIsTimerActive(xDeteccionTimer)) {
						xTimerStop( xDeteccionTimer, 0 );
					}
					if (xTimerIsTimerActive(xReactionTimer)) {
						xTimerStop( xReactionTimer, 0 );
					}
					data.mode = SYSTEM_ON;
				}
				else if (data.a_value < BRAKEMIN && !xTimerIsTimerActive(xReactionTimer))
					/* If timer of reaction is expired go emergency */
					data.mode = SYSTEM_EMERGENCY_BRAKE;
				break;

			case SYSTEM_EMERGENCY_BRAKE:
				if (IN_GetStatusSignal(ACK_WARNING))
					/* Go initial state */
					data.mode = SYSTEM_ON;
				break;
			}
		}
		xSemaphoreGive( xMutex );

		/* About a 10ms delay here */
		Delay_Ms(EXEC_STATUS, TIME_TASK);
	}
}

/* callback function of the Reaction timer */
static void vReactionTimerCallback( xTimerHandle xTimer )
{
	xTimerStop( xReactionTimer, 0 );
}

/* callback function of the Deteccion timer */
static void vDeteccionTimerCallback( xTimerHandle xTimer )
{
	xTimerStop( xDeteccionTimer, 0 );
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

int main(void)
{
	prvSetupHardware();

	/* Create the queue. */
	xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( char[WIFLY_SIZE] ) );

	if( xQueue != NULL )
	{
		vSemaphoreCreateBinary(xMutex);

		/* Only create the tasks if the semaphore was created successfully. */
		if( xMutex != NULL )
		{
			/* Start tasks  */
			xTaskCreate( vWIFLYTask, ( signed char * ) "WIFLY",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 10UL),
					(xTaskHandle *) NULL);
			xTaskCreate( vDEBUGTask, ( signed char * ) "DEBUG",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 5UL),
					(xTaskHandle *) NULL);
			xTaskCreate( vMAINTask, ( signed char * ) "MAIN",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 1UL),
					(xTaskHandle *) NULL);
			xTaskCreate( vADCTask, ( signed char * ) "ADC",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 1UL),
					(xTaskHandle *) NULL);
			xTaskCreate( vINTask, ( signed char * ) "INs",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 1UL),
					(xTaskHandle *) NULL);
			xTaskCreate( vOUTTask, ( signed char * ) "OUTs",
					configMINIMAL_STACK_SIZE * 2, NULL, (tskIDLE_PRIORITY + 1UL),
					(xTaskHandle *) NULL);

			/* Reaction Timer */
			xReactionTimer = xTimerCreate( ( signed char * ) "Reaction Timer",		// Just a text name, not used by the kernel.
											REACTION_TIME,     						// The timer period in ticks.
											pdFALSE,         						// The timers will auto-reload themselves when they expire.
											( void * ) 0,     						// The ID is not used, so can be set to anything.
											vReactionTimerCallback					// The callback function of the timer.
											);

			/* Deteccion Timer */
			xDeteccionTimer = xTimerCreate( ( signed char * ) "Deteccion Timer",	// Just a text name, not used by the kernel.
											DETECTION_TIME,     					// The timer period in ticks.
											pdFALSE,         						// The timers will auto-reload themselves when they expire.
											( void * ) 1,     						// The ID is not used, so can be set to anything.
											vDeteccionTimerCallback					// The callback function of the timer.
											);

			/* Start the tasks running. */
			vTaskStartScheduler();
		}
	}

	/* Should never arrive here */
	return 1;
}

